# stix2gen
# Copyright (c) 2021-2022 Claudio Guarnieri
# Please check the file 'LICENSE' for copying permission.

import validators
from stix2.v21 import Indicator

IOC_TYPE_SHA256 = "sha256"
IOC_TYPE_DOMAIN = "domain"
IOC_TYPE_EMAIL = "email"
IOC_TYPE_IPV4 = "ipv4"
IOC_TYPE_IPV6 = "ipv6"
IOC_TYPE_APP_ID = "app_id"

IOC_MODELS = [
    {
        "type": IOC_TYPE_SHA256,
        "validator": validators.sha256,
        "stix2_pattern": "[file:hashes.sha256='{value}']"
    },
    {
        "type": IOC_TYPE_DOMAIN,
        "validator": validators.domain,
        "stix2_pattern": "[domain-name:value='{value}']",
    },
    {
        "type": IOC_TYPE_EMAIL,
        "validator": validators.email,
        "stix2_pattern": "[email-addr:value='{value}']",
    },
    {
        "type": IOC_TYPE_IPV4,
        "validator": validators.ipv4,
        "stix2_pattern": "[ipv4-addr:value='{value}']",
    },
    {
        "type": IOC_TYPE_IPV6,
        "validator": validators.ipv6,
        "stix2_pattern": "[ipv6-addr:value='{value}']",
    },
    {
        "type": IOC_TYPE_APP_ID,
        "validator": None,
        "stix2_pattern": "[app:id='{value}']",
    }
]


class IOC(object):

    def __init__(self, ioc, ioc_type=None):
        self.ioc = ioc
        self.ioc_type = ioc_type
        self.stix2_pattern = None
        if self.ioc_type:
            for ioc_model in IOC_MODELS:
                if ioc_model["type"] != self.ioc_type:
                    continue

                self.stix2_pattern = ioc_model.get("stix2_pattern")
                break

    def clean(self):
        self.ioc = self.ioc.strip().replace("[.]", ".").replace("[@]", "@")

    def detect(self):
        for ioc_model in IOC_MODELS:
            if not ioc_model.get("validator"):
                continue

            if ioc_model["validator"](self.ioc):
                self.ioc_type = ioc_model.get("type")
                self.stix2_pattern = ioc_model.get("stix2_pattern")
                return

    def stix2(self):
        if not self.ioc_type:
            return None

        return Indicator(indicator_types=["malicious-activity"],
                         pattern_type="stix",
                         pattern=self.stix2_pattern.format(value=self.ioc))



# stix2gen
# Copyright (c) 2021-2022 Claudio Guarnieri
# Please check the file 'LICENSE' for copying permission.

import sys

import click
from stix2.v21 import Bundle, Malware, Relationship

from .ioc import IOC, IOC_TYPE_APP_ID


@click.group()
def cli():
    pass


@cli.command()
@click.option("--malware-name", help="Malware name", required=True)
@click.option("--malware-desc", help="Malware descrition")
@click.option("--app-ids-file", help="App IDs File", type=click.File("r"))
@click.option("--source-file", help="File", type=click.File("r"),
              default=sys.stdin)
def cli(malware_name, malware_desc, app_ids_file, source_file):
    objects = []

    malware = Malware(name=malware_name, description=malware_desc,
                      is_family=True)
    objects.append(malware)

    for line in source_file.readlines():
        ioc = IOC(line)
        ioc.clean()

        if not ioc.ioc:
            continue

        ioc.detect()

        if not ioc.ioc_type:
            continue

        stix2_obj = ioc.stix2()
        objects.append(stix2_obj)
        objects.append(Relationship(stix2_obj, "indicates", malware))

    if app_ids_file:
        for line in app_ids_file.readlines():
            ioc = IOC(ioc=line, ioc_type=IOC_TYPE_APP_ID)
            ioc.clean()

            stix2_obj = ioc.stix2()
            objects.append(stix2_obj)
            objects.append(Relationship(stix2_obj, "indicates", malware))

    bundle = Bundle(objects=objects)
    sys.stdout.write(bundle.serialize(pretty=True))
